"""
Settings for django-expanded-test-cases UnitTesting.
"""

# System Imports.
import os, sys


BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
SECRET_KEY = 'test-secret-key'


# Determine if channels is installed.
try:
    from channels.testing import ChannelsLiveServerTestCase

    # If we made it this far, channels is installed.
    CHANNELS_PACKAGE_INSTALLED = True
except ModuleNotFoundError:
    # Failed to import channels. Assume not installed.
    CHANNELS_PACKAGE_INSTALLED = False


INSTALLED_APPS = (
    'django_expanded_test_cases',

    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
)
if CHANNELS_PACKAGE_INSTALLED:
    INSTALLED_APPS += ('channels', 'daphne')
ASGI_APPLICATION = 'tests.asgi.application'


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
        'TEST': {
            'NAME': os.path.join(BASE_DIR, 'db_test.sqlite3'),
        },
    },
}


MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]


ROOT_URLCONF = 'tests.urls'
LOGIN_URL = 'django_expanded_test_cases:login'
STATIC_URL = '/static/'
USE_TZ = True


TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]


# Suppress or show testcase debug printout, based on UnitTest execution method.
if 'pytest' in sys.modules:
    # Running Pytest env.
    # Pytest only shows console output on test failure, so we want it on.
    DJANGO_EXPANDED_TESTCASES_DEBUG_PRINT = True
else:
    # Running other testing env (mostly likely "django manage.py test").
    # manage.py shows all console output always, even on success.
    # So we want it off to avoid information overload and spam.
    DJANGO_EXPANDED_TESTCASES_DEBUG_PRINT = False


# Extra definable package settings.
# Here for personal reference at later point, for documentation and such.
# DJANGO_EXPANDED_TESTCASES_ALLOW_MESSAGE_PARTIALS = True
# DJANGO_EXPANDED_TESTCASES_MATCH_ALL_CONTEXT_MESSAGES = False

# Valid, supported selenium browser options.
# SELENIUM_TEST_BROWSER = 'chrome'
# SELENIUM_TEST_BROWSER = 'chromium'
# SELENIUM_TEST_BROWSER = 'firefox'

# Support for changing console color output.
# from colorama import Back, Fore, Style
# DJANGO_EXPANDED_TESTCASES_OUTPUT_EXPECTED_MATCH = '{0}{1}{2}'.format(Fore.CYAN, Back.RESET, Style.NORMAL)
# DJANGO_EXPANDED_TESTCASES_OUTPUT_EXPECTED_ERROR = '{0}{1}{2}'.format(Fore.BLACK, Back.CYAN, Style.NORMAL)
# DJANGO_EXPANDED_TESTCASES_OUTPUT_ACTUALS_MATCH = '{0}{1}{2}'.format(Fore.MAGENTA, Back.RESET, Style.NORMAL)
# DJANGO_EXPANDED_TESTCASES_OUTPUT_ACTUALS_ERROR = '{0}{1}{2}'.format(Fore.BLACK, Back.MAGENTA, Style.NORMAL)



"""
Testing class for generalized logic.
"""

# Third-Party Imports.
from django.test import TestCase

# Internal Imports.
from django_expanded_test_cases.constants import ETC_OUTPUT_EMPHASIS_COLOR, ETC_OUTPUT_ERROR_COLOR
from django_expanded_test_cases.mixins import CoreTestCaseMixin


class BaseTestCase(TestCase, CoreTestCaseMixin):
    """Generalized testing functionality. Builds upon Django's default TestCase class."""

    @classmethod
    def setUpClass(cls, debug_print=None):
        # Run parent setup logic.
        super().setUpClass()

        # Also call CoreMixin setup logic.
        cls.set_up_class(debug_print=debug_print)

    @classmethod
    def setUpTestData(cls):
        """"""
        # Initialize default data models.
        cls.set_up_test_data()

    def setUp(self):
        # Run parent setup logic.
        super().setUp()

        self._error_displayed = False

    def subTest(self, *args, **kwargs):
        # Call CoreMixin logic.
        self.sub_test()

        # Run parent logic.
        return super().subTest(*args, **kwargs)

    def _handle_test_error(self, err):
        """
        Handling for errors in UnitTesting.

        Mostly for ensuring consistent output on assertion failure.
        """
        # Only handle error if an earlier function call has not yet done so.
        # This helps prevent calling this logic multiple times on error (and thus spamming console output),
        # regardless of order of testing functions.
        # We also skip output such as when the logger is disabled.
        # This is likely due to temporary checks failing, and not a legitimate error/failure.
        if not self._error_displayed:
            # Print error to both logging and standard console output.
            self._debug_print(
                '{0} {1} UnitTesting {2} {0}'.format(
                    ('=' * 10),
                    self.__class__.__name__,
                    type(err).__name__,
                ),
                fore=ETC_OUTPUT_ERROR_COLOR,
                style=ETC_OUTPUT_EMPHASIS_COLOR,
            )
            self._debug_print('{0}\n\n'.format(str(err)))

            # Save that we have output error.
            self._error_displayed = True

    # region Default Test Function Overrides

    def fail(self, *args, **kwargs):
        try:
            return super().fail(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertEqual(self, *args, **kwargs):
        try:
            return super().assertEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    # NOTE: Depreciated as of Python3.2.
    def assertEquals(self, *args, **kwargs):
        try:
            return super().assertEquals(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertNotEqual(self, *args, **kwargs):
        try:
            return super().assertNotEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    # NOTE: Depreciated as of Python3.2.
    def assertNotEquals(self, *args, **kwargs):
        try:
            return super().assertNotEquals(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertTrue(self, *args, **kwargs):
        try:
            return super().assertTrue(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertFalse(self, *args, **kwargs):
        try:
            return super().assertFalse(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertIs(self, *args, **kwargs):
        try:
            return super().assertIs(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertIsNot(self, *args, **kwargs):
        try:
            return super().assertIsNot(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertNot(self, *args, **kwargs):
        try:
            return super().assertNot(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertIsNone(self, *args, **kwargs):
        try:
            return super().assertIsNone(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertIsNotNone(self, *args, **kwargs):
        try:
            return super().assertIsNotNone(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertIn(self, *args, **kwargs):
        try:
            return super().assertIn(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertNotIn(self, *args, **kwargs):
        try:
            return super().assertNotIn(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertIsInstance(self, *args, **kwargs):
        try:
            return super().assertIsInstance(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertNotIsInstance(self, *args, **kwargs):
        try:
            return super().assertNotIsInstance(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertRaises(self, *args, **kwargs):
        try:
            return super().assertRaises(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertRaisesRegex(self, *args, **kwargs):
        try:
            return super().assertRaisesRegex(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    # Depreciated as of Python 3.2.
    def assertRaisesRegexp(self, *args, **kwargs):
        try:
            return super().assertRaisesRegexp(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertWarns(self, *args, **kwargs):
        try:
            return super().assertWarns(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertWarnsRegex(self, *args, **kwargs):
        try:
            return super().assertWarnsRegex(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertWarnsMessage(self, *args, **kwargs):
        try:
            return super().assertWarnsMessage(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertLogs(self, *args, **kwargs):
        try:
            return super().assertLogs(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    # NOTE: New as of Python 3.10.
    def assertNoLogs(self, *args, **kwargs):
        try:
            return super().assertNoLogs(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertAlmostEqual(self, *args, **kwargs):
        try:
            return super().assertAlmostEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    # Depreciated as of Python 3.2.
    def assertAlmostEquals(self, *args, **kwargs):
        try:
            return super().assertAlmostEquals(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertNotAlmostEqual(self, *args, **kwargs):
        try:
            return super().assertNotAlmostEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    # Depreciated as of Python 3.2.
    def assertNotAlmostEquals(self, *args, **kwargs):
        try:
            return super().assertNotAlmostEquals(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertGreater(self, *args, **kwargs):
        try:
            return super().assertGreater(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertGreaterEqual(self, *args, **kwargs):
        try:
            return super().assertGreaterEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertLess(self, *args, **kwargs):
        try:
            return super().assertLess(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertLessEqual(self, *args, **kwargs):
        try:
            return super().assertLessEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertRegex(self, *args, **kwargs):
        try:
            return super().assertRegex(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    # Depreciated as of Python 3.2.
    def assertRegexpMatches(self, *args, **kwargs):
        try:
            return super().assertRegexpMatches(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertNotRegex(self, *args, **kwargs):
        try:
            return super().assertNotRegex(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    # Depreciated as of Python 3.2.
    def assertNotRegexpMatches(self, *args, **kwargs):
        try:
            return super().assertNotRegexpMatches(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertCountEqual(self, *args, **kwargs):
        try:
            return super().assertCountEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertMultiLineEqual(self, *args, **kwargs):
        try:
            return super().assertMultiLineEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertSequenceEqual(self, *args, **kwargs):
        try:
            return super().assertSequenceEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertListEqual(self, *args, **kwargs):
        try:
            return super().assertListEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertTupleEqual(self, *args, **kwargs):
        try:
            return super().assertTupleEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertSetEqual(self, *args, **kwargs):
        try:
            return super().assertSetEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    def assertDictEqual(self, *args, **kwargs):
        try:
            return super().assertDictEqual(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    # Depreciated as of Python ???.
    def assertDictContainsSubset(self, *args, **kwargs):
        try:
            return super().assertDictContainsSubset(*args, **kwargs)
        except Exception as err:
            # If any error occurs, this makes sure it displays to console as the most recent output.
            self._handle_test_error(err)
            raise err

    # endregion Default Test Function Overrides


# Define acceptable imports on file.
__all__ = [
    'BaseTestCase',
]



"""
Universal mixin logic for live server instances, such as through selenium.
Useful for testing things like JavaScript logic.

Tends to take longer to test. So consider using IntegrationTestCase instead, when possible.
"""

# System Imports.
import time

# Third-Party Imports.
from selenium import webdriver

# Internal Imports.
from django_expanded_test_cases.constants import (
    ETC_INCLUDE_RESPONSE_DEBUG_URL,
    ETC_INCLUDE_RESPONSE_DEBUG_CONTENT,
    ETC_RESPONSE_DEBUG_CONTENT_COLOR,
    ETC_OUTPUT_EMPHASIS_COLOR,
)
from django_expanded_test_cases.mixins.response_mixin import ResponseTestCaseMixin


class LiveServerMixin(ResponseTestCaseMixin):
    """Universal logic for all selenium LiveServer test cases."""

    # region Utility Functions

    def create_driver(self):
        """Creates new browser manager instance."""

        # Create instance, based on selected driver type.
        if self._browser == 'chrome':
            driver = webdriver.Chrome(service=self._service, options=self._options)
        elif self._browser == 'firefox':
            driver = webdriver.Firefox(service=self._service, options=self._options)
        else:
            raise ValueError('Unknown browser "{0}".'.format(self._browser))

        # Make class aware of window.
        self._driver_set.append(driver)

        # Set to check page for 5 seconds before giving up on element.
        driver.implicitly_wait(5)

        return driver

    def get_driver(self):
        """Returns first driver off of driver stack, or creates new one if none are present."""
        try:
            return self._driver_set[-1]
        except IndexError:
            return self.create_driver()

    def close_driver(self, driver):
        """Closes provided browser manager instance.

        :param driver: Driver manager object to close.
        """
        # Remove reference in class.
        self._driver_set.remove(driver)

        # Close window.
        driver.quit()

    def close_all_drivers(self):
        """Closes all open browser manager instances."""
        while len(self._driver_set) > 0:
            self.close_driver(self._driver_set[0])

    def open_new_window(self, driver):
        """Opens a new window for the provided driver.

        :param driver: Driver manager object to generate window for.
        :return: New focus window.
        """
        # Open blank new window.
        driver.switch_to.new_window('window')

        # Switch to recently created window.
        return self.switch_to_window_at_index(driver, len(driver.window_handles) - 1)

    def open_new_tab(self, driver):
        """Opens a new window for the provided driver.

        :param driver: Driver manager object to generate window for.
        :return: New focus window.
        """
        # Open blank new window.
        driver.switch_to.new_window('tab')

        # Switch to recently created window.
        return self.switch_to_window_at_index(driver, len(driver.window_handles) - 1)

    def close_window_at_index(self, driver, window_index):
        """Closes a window at specific index for the provided driver.

        :param driver: Driver manager object containing the desired window.
        :param window_index: Index of window to close.
        """
        # Attempt to get window at specified driver index.
        try:
            focus_window = driver.window_handles[window_index]
        except IndexError:
            err_msg = 'Attempted to close to window of index "{0}", but driver only has "{1}" windows open.'.format(
                window_index,
                len(driver.window_handles)
            )
            raise IndexError(err_msg)

        # Close window.
        self.switch_to_window_at_index(driver, window_index)
        driver.execute_script('window.close();')

    def switch_to_window_at_index(self, driver, window_index):
        """Sets window at specific driver/index to be the current focus.

        :param driver: Driver manager object containing the desired window.
        :param window_index: Index of window to switch to.
        :return: New focus window.
        """
        # Attempt to get window at specified driver index.
        try:
            focus_window = driver.window_handles[window_index]
        except IndexError:
            err_msg = 'Attempted to switch to window of index "{0}", but driver only has "{1}" windows open.'.format(
                window_index,
                len(driver.window_handles)
            )
            raise IndexError(err_msg)

        # Switch window to be focused.
        driver.switch_to.window(focus_window)

        # Return newly switched window.
        return focus_window

    def sleep_browser(self, seconds):
        """Halts browser for provided number of seconds.

        Useful for visually verifying browser state, when trying to debug tests.
        """
        time.sleep(seconds)

    def is_webdriver(self, obj):
        """Verifies if object matches set on known web driver types."""

        # Import all known drivers for type checking.
        from selenium.webdriver.chrome.webdriver import WebDriver as ChromeDriver
        from selenium.webdriver.chromium.webdriver import RemoteWebDriver as ChromiumDriver
        from selenium.webdriver.edge.webdriver import WebDriver as EdgeDriver
        from selenium.webdriver.firefox.webdriver import WebDriver as FireFoxDriver
        from selenium.webdriver.ie.webdriver import WebDriver as IeDriver
        from selenium.webdriver.safari.webdriver import WebDriver as SafariDriver

        # Check against known drivers.
        if (
            isinstance(obj, ChromeDriver)
            or isinstance(obj, ChromiumDriver)
            or isinstance(obj, EdgeDriver)
            or isinstance(obj, FireFoxDriver)
            or isinstance(obj, IeDriver)
            or isinstance(obj, SafariDriver)
        ):
            # Matched a known driver.
            return True
        else:
            # Did not match any known drivers.
            return False

    def show_debug_data(self, content):
        """"""
        if ETC_INCLUDE_RESPONSE_DEBUG_URL:
            self.show_debug_url(None)
        if ETC_INCLUDE_RESPONSE_DEBUG_CONTENT:
            self.show_debug_content(content)

    def show_debug_url(self, url):
        """Prints debug url output."""

        # Skip if no Url is present.
        if url is None or str(url).strip() == '':
            url = self.current_url
        if url is None or str(url).strip() == '':
            return
        else:
            return super().show_debug_url(url)

    def show_debug_content(self, content):
        """Prints debug response page output."""

        self._debug_print()
        self._debug_print(
            '{0} {1} {0}'.format('=' * 10, 'response.content'),
            fore=ETC_RESPONSE_DEBUG_CONTENT_COLOR,
            style=ETC_OUTPUT_EMPHASIS_COLOR,
        )

        # Print out data, if present.
        if content:
            self._debug_print(self.get_minimized_response_content(content))
            # self._debug_print(content)
            self._debug_print()

    # endregion Utility Functions

    # region Html Search Functions

    def find_elements_by_tag(self, content, element):
        """Finds all HTML elements that match the provided element tag.

        :param content: Content to search through.
        :param element: Html element to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_elements_by_tag(content, element)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_element_by_tag(self, content, element):
        """Finds first HTML element that matches the provided element tag.

        :param content: Content to search through.
        :param element: Html element to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_element_by_tag(content, element)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_elements_by_id(self, content, element_id):
        """Finds all HTML elements that match the provided id.

        :param content: Content to search through.
        :param element_id: Element id to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_elements_by_id(content, element_id)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_element_by_id(self, content, element_id):
        """Finds first HTML element that matches the provided id.

        :param content: Content to search through.
        :param element_id: Element id to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_element_by_id(content, element_id)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_elements_by_class(self, content, css_class):
        """Finds all HTML elements that match the provided css class.

        :param content: Content to search through.
        :param css_class: Css class to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_elements_by_class(content, css_class)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_element_by_class(self, content, css_class):
        """Finds first HTML element that matches the provided css class.

        :param content: Content to search through.
        :param css_class: Css class to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_element_by_class(content, css_class)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_elements_by_css_selector(self, content, css_selector):
        """Finds all HTML elements that match the provided css selector.

        :param content: Content to search through.
        :param css_selector: Css selector to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_elements_by_css_selector(content, css_selector)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_element_by_css_selector(self, content, css_selector):
        """Finds first HTML element that matches the provided css selector.

        :param content: Content to search through.
        :param css_selector: Css selector to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_element_by_css_selector(content, css_selector)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_elements_by_data_attribute(self, content, data_attribute, data_value):
        """Finds all HTML elements that match the provided data attribute and data value.

        :param content: Content to search through.
        :param data_attribute: The key of the data attribute to search for.
        :param data_value: The value of the data attribute to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_elements_by_data_attribute(content, data_attribute, data_value)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_element_by_data_attribute(self, content, data_attribute, data_value):
        """Finds first HTML element that matches the provided data attribute and data value.

        :param content: Content to search through.
        :param data_attribute: The key of the data attribute to search for.
        :param data_value: The value of the data attribute to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_element_by_data_attribute(content, data_attribute, data_value)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_elements_by_name(self, content, element_name):
        """Finds all HTML elements that match the provided name attribute.

        :param content: Content to search through.
        :param element_name: Element name to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_elements_by_name(content, element_name)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_element_by_name(self, content, element_name):
        """Finds first HTML element that matches the provided name attribute.

        :param content: Content to search through.
        :param element_name: Element name to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_element_by_name(content, element_name)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_elements_by_link_text(self, content, link_text):
        """Finds all HTML elements that match the provided link text.

        :param content: Content to search through.
        :param link_text: Link text to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_elements_by_link_text(content, link_text)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    def find_element_by_link_text(self, content, link_text):
        """Finds first HTML element that matches the provided link text.

        :param content: Content to search through.
        :param link_text: Link text to search for.
        """
        self.current_url = None

        # Handle if webdriver was provided.
        # Otherwise assume was standard "page content".
        if self.is_webdriver(content):
            self.current_url = content.current_url
            content = content.page_source

        try:
            # Return original parent call with correct variables.
            return super().find_element_by_link_text(content, link_text)
        except Exception as err:
            self.show_debug_data(content)
            raise err

    # endregion Html Search Functions


# Define acceptable imports on file.
__all__ = [
    'LiveServerMixin',
]



"""
Testing URL configuration for django-expanded-test-cases project.

Mocks being an "app" urls.py file.
"""

# Third-Party Imports.
from django.urls import path

# Internal Imports.
from tests import views


app_name = 'django_expanded_test_cases'
urlpatterns = [
    # Simple test views.
    path('login/', views.login, name='login'),
    path('views/one-message/', views.view_with_one_message, name='response-with-one-message'),
    path('views/two-messages/', views.view_with_two_messages, name='response-with-two-messages'),
    path('views/three-messages/', views.view_with_three_messages, name='response-with-three-messages'),
    path('views/repeating-elements/', views.view_with_repeating_elements, name='response-with-repeating-elements'),
    path('views/<int:id>/<str:name>/', views.view_with_args, name='response-with-args'),


    # Template response views.
    path('template-response/index/', views.template_response_index, name='template-response-index'),
    path('template-response/messages/', views.template_response_with_three_messages, name='template-response-messages'),
    path('template-response/<int:id>/<str:name>/', views.template_response_with_args, name='template-response-with-args'),

    # Model test views.
    path('user/detail/<int:pk>/', views.user_detail, name='user-detail'),

    # Redirect views.
    path('redirect/index/', views.redirect_to_index, name='redirect-to-index'),
    path('redirect/with_args/<int:id>/<str:name>/', views.redirect_with_args, name='redirect-with-args'),

    # Index view.
    path('', views.index, name='index'),
]
